BeHOLd.ClassicalHOL.Patterns (behold v1.1.3)

Defines patterns on terms with respect to the definitions introduced in BeHOLd.ClassicalHOL.Definitions. Note that this should only be used for pattern matching and not for term construction!

Summary

Functions

Pattern to match the conjunction of two terms.

Pattern to match the disjunction of two terms.

Pattern to match equiality of two terms, ignoring the type.

Pattern to match the equivalence of two terms.

Pattern to match the existential quantification of a given predicate, ignoring the type.

Pattern to match the implication of two terms, where the first term implies the second.

Pattern to match a negated term.

Pattern to match equality of two terms with respect to their type.

Pattern to match the existential quantification of a given predicate with respect to its type.

Pattern to match the universal quantification of a given predicate with respect to its type.

Pattern to match the universal quantification of a given predicate ignoring the type.

Functions

conjunction(p, q)

(macro)

Pattern to match the conjunction of two terms.

disjunction(p, q)

(macro)

Pattern to match the disjunction of two terms.

equality(a, b)

(macro)

Pattern to match equiality of two terms, ignoring the type.

equivalence(p, q)

(macro)

Pattern to match the equivalence of two terms.

existential_quantification(pp)

(macro)
@spec existential_quantification(HOL.Data.hol_term()) :: HOL.Data.hol_term()

Pattern to match the existential quantification of a given predicate, ignoring the type.

implication(p, q)

(macro)

Pattern to match the implication of two terms, where the first term implies the second.

negated(term)

(macro)
@spec negated(HOL.Data.hol_term()) :: HOL.Data.hol_term()

Pattern to match a negated term.

typed_equality(a, b, t)

(macro)

Pattern to match equality of two terms with respect to their type.

typed_existential_quantification(pp, t)

(macro)
@spec typed_existential_quantification(HOL.Data.hol_term(), HOL.Data.type()) ::
  HOL.Data.hol_term()

Pattern to match the existential quantification of a given predicate with respect to its type.

typed_universal_quantification(pp, t)

(macro)
@spec typed_universal_quantification(HOL.Data.hol_term(), HOL.Data.type()) ::
  HOL.Data.hol_term()

Pattern to match the universal quantification of a given predicate with respect to its type.

universal_quantification(pp)

(macro)
@spec universal_quantification(HOL.Data.hol_term()) :: HOL.Data.hol_term()

Pattern to match the universal quantification of a given predicate ignoring the type.